	INCLUDE <P18F1320.INC>
;	PWM MULLER MOTOR DRIVER CODE VERSION 1.1
;Version 1.1 added ability for Modulated output pulse
;Version 1.0 release 13/08/11
;	CONFIG   OSC = INTIO2, PWRT = ON, WDT = OFF, MCLRE = OFF, LVP = OFF
	CBLOCK 	0x00 		;start of general purpose ram
	STATUS_TEMP			;TEMP STORAGE OF STATUS REGISTER DURING INTERRUPT SERVICE ROUTINE
	WREG_TEMP			;TEMP STORAGE OF WREG DURING INTERRUPT SERVICE ROUTINE
	BSR_TEMP			;TEMP STORAGE OF BSR DURING ISR	
	ROTOR_COUNT			;COUNTS THE MAGNET PASSES 0-7 FOR A 8 MAGNET ROTOR, GIVES ME A 1 FULL ROTATION COUNT.
	RPM_LOW				;RPM TIME LOW BYTE
	RPM_HIGH			;RPM TIME HIGH BYTE
	RPM_MAX_HIGH		;MAX TIMER VALUE HIGH BYTE TO STOP FET FROM BURNING UP
	RPM_MAX_LOW			;MAX TIMER VALUE LOW BYTE TO STOP FET FROM BURNING UP
	PWM_ON_HIGH			;PWM ON HIGH BYTE FOR PRELOADING TIMER
	PWM_ON_LOW			;PWM ON LOW BYTE FOR PRELOADING TIMER
	PWM_ERROR			; IF 1 THEN THE HALL SWITCHED OFF THE FETGATE AND OUR PULSE WIDTH OVERFLOWED
	RPM_SETPOINT_HIGH	;TARGET RPM TIMER VALUE HIGH BYTE ADJUSTED BY USING UP/DOWN KEYS
	RPM_SETPOINT_LOW	;TARGET RPM TIMER VALUE LOW BYTE ADJUSTED BY USING UP/DOWN KEYS
	D1					;DELAY VARIABLE 1
	D2					;DELAY VARIABLE 2
	D3					;DELAY VARIABLE 3
	ENDC
;*******************************************
; MOTOR PRESET PARAMETERS				   
;*******************************************

RPM_SP_PRE_HIGH	EQU 0X41	;TARGET RPM VALUE FOR TIMER HIGH BYTE 41A3 IS ABOUT 225 RPM
RPM_SP_PRE_LOW	EQU 0XA3	;TARGET RPM VALUE FOR TIMER LOW BYTE THIS IS OUR STARTING RPM. SLOWISH TO START

RPM_SP_MIN_HIGH	EQU 0X04	;USED TO CHECK OUR MINIMUM TIMER BOUNDARY, SETS THE MIN VALUE FOR THE UP BUTTON ** SMALLER VALUE=FASTER ROTOR
RPM_SP_MIN_LOW	EQU 0XED	;SET FOR ABOUT 3000 RPM

RPM_SP_MAX_HIGH	EQU 0X94	;USED TO CHECK OUR MAXIMUM TIMER BOUNDARY, SETS THE MAX VALUE FOR THE DOWN BUTTON **LARGER VALUE=SLOWER ROTOR
RPM_SP_MAX_LOW	EQU 0X1A	;SET FOR ABOUT 100 RPM
;-------------------------------------------------------------------------------------------------------------
;PWM IS SET UP FOR 40kHZ 50% DUTY IF YOU WANT TO ALTER THIS THEN GOTO THIS WEB SITE AND ALTER THE VALUES BELOW
;http://www.micro-examples.com/public/microex-navig/doc/097-pwm-calculator.html
PWM				EQU 0			;THIS SWITCHES OUR DRIVER TO USE PWM INSTEAD OF PLAIN ON/OFF 1=PWM ON, 0=PWM OFF
PWM_PR2			EQU B'00110001' ;PWM PERIOD REGISTER VALUE
PWM_T2CON		EQU B'00000100' ;SET UP TIMER 2 AND PRESCALE
PWM_CCPR1L		EQU B'00011000' ;PWM DUTY CYCLE
PWM_CCP1CON		EQU B'00111100' ;ONLY 1 OUTPUT USED FOR PWM, ACTIVE HIGH PULSE
;--------------------------------------------------------------------------------------------------------------

PWM_ON_PRE_HIGH EQU 0XF5	;INITIAL START PWM ON PERIOD HIGH BYTE
PWM_ON_PRE_LOW	EQU 0X00	;INITIAL START PWM ON PERIOD LOW BYTE

PWM_ON_MAX_HIGH EQU 0XE0	;MAX PWM ON PERIOD HIGH BYTE THIS WOULD BE A LOWER NUMBER AS IT RELATES TO THE TIMER VALUE
PWM_ON_MAX_LOW	EQU 0X00	;MAX PWM ON PERIOD LOW BYTE ABOUT 4.2mS FFFF-E000=8191 = 4.09mS + OVERHEAD PROCESSING

PWM_ON_MIN_HIGH EQU 0XFE	;MIN PWM ON PERIOD HIGH BYTE THIS WOULD BE A LARGER NUMBER AS IT RELATES TO THE TIMER VALUE
PWM_ON_MIN_LOW	EQU 0X00	;MIN PWM ON PERIOD LOW BYTE THIS IS ABOUT 400uS FFFF-FE00=511 = 255uS + OVERHEAD PROCESSING THE TIMER IS RUNNING AT 8MHZ/4 TICS

NUM_MAGNETS		EQU 8		;NUMBER OF MAGNETS ON ROTOR THIS ALLOWS US TO CHANGE OUR ROTOR MAGNET NUMBER
;*******************************************
;***VARIABLE EQUATES FOR PORT DESTINATION***
;*******************************************
;PORT A BITS FUNCTION DEFINITION
DOWNKEY		EQU 6	;UP KEY
UPKEY		EQU 7	;DOWN KEY
;PORT B BITS FUNCTION DEFINITION NOTE RB4-RB7 ARE SETUP FOR INTERRUPT ON LEVEL CHANGE
MENUKEY		EQU 1	;MENU KEY
FETGATE		EQU 3	;MOTOR DRIVE FET GATE
HALL		EQU	6	;HALL SENSOR INPUT

;*********************************************
	ORG		0x0000
   	GOTO	MAIN
	ORG		0X0008
	GOTO	LOW_INTERUPT
	ORG		0X0018
	GOTO	LOW_INTERUPT
HIGH_INTERUPT
	RETFIE
;*******************************************************************************************************************;
; LOW INTERRUPT PRIORITY LEVEL SERVICE ROUTINE, SERVICES TIMER0 AND TIMER1 ROUTINES AND FET ON/OFF PERIOD			;
; TIMER 0 IS USED TO TIME n(8) MAGNET PASSES OR 1 REVOLUTION OF THE ROTOR, THE FASTER THE ROTOR THE LOWER THE VALUE ;
; TIMER 1 IS USED TO TIME THE ON PERIOD FOR THE FET DRIVE, IF THE ROTOR RPM IS LESS THAN RPM SETPOINT THEN THE ON   ;
; PERIOD IS INCREASED UNTIL WE HAVE REACHED OUR TARGET SETPOINT, LIKEWISE IF THE ROTOR IS TOO FAST WE DECREASE      ;
; IF THE HALL REVERTS TO A OFF HIGH STATE THE FET IS ALSO TURNED OFF                                                ;
;*******************************************************************************************************************;
LOW_INTERUPT
	BCF		INTCON,GIE
	MOVFF	STATUS,STATUS_TEMP;SAVE STATUS REGISTER FOR RESTORING AFTER ISROUTINE
	MOVFF	WREG,WREG_TEMP	;SAVE WREG FOR RESTORING AFTER ISR
	MOVFF	BSR,BSR_TEMP	;SAME FOR BSR
isr_next
	BTFSS	PIR1,TMR1IF		;CHECK FOR PWM TIMER1 OVERFLOW
	GOTO 	check_pins		;IF IT WAS NOT TIMER1 THAT INTERRUPTED THEN GO CHECK PIN CHANGE
;IF PWM MODE IS OFF THEN JUST SWITCH FET OFF
 IF PWM==0
	BCF		PORTB,FETGATE	;YES TIMER1 OVERFLOWED SO TURN OFF FET GATE
 ELSE
;IF PWM MODE IS ON THEN SWITCH PWM OFF
	BSF		ECCPAS,ECCPASE	;SHUTDOWN PWM UNTIL WE NEED IT
 ENDIF
	BCF		T1CON,TMR1ON	;DISABLE TIMER 1
	BCF		PIR1,TMR1IF		;CLEAR TIMER 1 OVERFLOW TO STOP RECURRING INTERRUPTS.	
check_pins
	BTFSS	INTCON,RBIF		;CHECK PIN CHANGE
	GOTO	isr_exit		;IF NOT EXIT
	BTFSS	PORTB,HALL		;TEST HALL BIT EXIT IF LOW WE WANT TO TURN ON THE FET FOR TIMER1 DURATION	
	GOTO	hall_low		;IF NOT EXIT
hall_high
	BTFSS	T1CON,TMR1ON	; CHECK TO SEE IF FET DRIVE IS STILL ON 
	GOTO	isr_exit		;WE WERE OK SO EXIT ISR
;IF PWM MODE IS OFF THEN JUST SWITCH FET OFF
 IF PWM==0
	BCF		PORTB,FETGATE	;TURN OFF FET GATE TO STOP DAMAGE
 ELSE
;IF PWM MODE IS ON THEN SWITCH PWM OFF
	BSF		ECCPAS,ECCPASE	;SHUTDOWN PWM UNTIL WE NEED IT
 ENDIF
	BSF		PWM_ERROR,0		;SET OVERFLOW ERROR FOR EXCESSIVE PWM ON PERIOD
	BCF		T1CON,TMR1ON	;DISABLE TIMER 1 AS IT IS NOW NOT NEEDED TO TIME ANY MORE DUE TO OVERFLOW PWM PERIOD 'ON STATE'
	GOTO	isr_exit
hall_low
;OK SO THE HALL HAS INDICATED IT IS ON LOGIC LOW AND SO WE NOW WANT TO SWITCH ON TIMER 1 TO TIME THE HIGH STATE MOTOR DRIVE PULSE WIDTH
	MOVFF	PWM_ON_HIGH,TMR1H ;TRANSFER INTO TIMER1 HIGH REGISTER
	MOVFF	PWM_ON_LOW,TMR1L  ;TRANSFER INTO TIMER1 LOW REGISTER
	BCF		PIR1,TMR1IF		;CLEAR TIMER 1 OVERFLOW
	BSF		T1CON,TMR1ON	;ENABLE TIMER 1
;IF PWM MODE IS OFF THEN JUST SWITCH FET ON
 IF PWM==0
	BSF		PORTB,FETGATE	;SWITCH ON FET
 ELSE
;IF PWM MODE IS ON THEN SWITCH PWM ON
	BCF		ECCPAS,ECCPASE	;STARTUP PWM AND DRIVE THE FET
 ENDIF
	INCF	ROTOR_COUNT,F
	MOVLW	NUM_MAGNETS		;PRELOAD F REG FOR CHECKING ROTOR COUNT OVERFLOW
	CPFSGT	ROTOR_COUNT		;HAVE WE HAD 8 MAGNETS PASS FOR 1 REV
	GOTO	isr_exit		;NOT SEEN A FULL ROTATION YET SO EXIT ISR
	CLRF	ROTOR_COUNT		;RESET ROTOR COUNT TO ZERO AS WE HAVE NOW SEEN 8 MAGNETS PASS
;HERE THE TIMER0 16 BIT VALUE NOW EQUALS THE TIME THE ROTOR TOOK FOR 1 FULL ROTATION, IF THE TIMER OVERFLOWED WE ARE REAL LOW REVS
;AND BELOW 30.52 RPM
	BTFSS	INTCON,TMR0IF	;CHECK FOR TIMER OVERFLOW LOW RPM BELOW 30.52
	GOTO	not_overflow	;DID NOT OVERFLOW GO DO CODE
	BCF		INTCON,TMR0IF	;CLEAR TIMER0 OVERFLOW
	MOVLW	0XFF
	MOVWF	RPM_LOW			;STORE LOW BYTE AS OVERFLOWED 'FFFF'
	MOVWF	RPM_HIGH		;STORE HIGH BYTE AS OVERFLOWED
	CLRF	TMR0H
	CLRF	TMR0L			;CLEAR TIMER 0
	GOTO 	isr_exit		;OUR WORK IS DONE SO EXIT INTERRUPT
not_overflow
	MOVFF	TMR0L,RPM_LOW	;GET RPM LOW BYTE AND STORE
	MOVFF	TMR0H,RPM_HIGH	;GET RPM HIGH BYTE AND STORE
	CLRF	TMR0H
	CLRF	TMR0L			;CLEAR TIMER 0
	BCF		INTCON,TMR0IF	;CLEAR TIMER0 OVERFLOW JUST INCASE IT'S OVERFLOWED SINCE READING AS IT'S FREEWHEELING
	BCF		INTCON,RBIF		;CLEAR INTERUPT FLAG PORT ON CHANGE
isr_exit
	MOVFF	STATUS_TEMP,STATUS;RESTORE STATUS REG SO NO REGISTERS WERE ALTERED DURING THE ISR
	MOVFF	WREG_TEMP,WREG	;RESTORE WREG SO NO REGISTERS WERE ALTERED DURING ISR EXECUTION
	MOVFF	BSR_TEMP,BSR	;SAME FOR BSR	
	BSF		INTCON,GIE
	RETFIE	
					
MAIN
;************************************************************************;
;			INIT PORTS DIRECTION AND INITIAL LEVELS                      ;
;*************************************************************************
	CLRF	PORTA			;CLEAR ALL PORTA BITS TO '0' LEVEL
	CLRF	PORTB			;CLEAR ALL PORTB BITS TO '0' OR LOGIC LOW
	CLRF	LATA			;CLEAR ALL PORTA BITS TO '0' LEVEL
	CLRF	LATB			;CLEAR ALL PORTB BITS TO '0' OR LOGIC LOW
	CLRF	RPM_HIGH		;RESET RPM COUNT HIGH BYTE
	CLRF	RPM_LOW			;CLEAR RPM COUNT LOW BYTE
	MOVLW	0X073			;SET OSC FREQ TO 8MHZ
	MOVWF	OSCCON
	MOVLW	0x07F			;TURN A/D CONVERTERS OFF AND CONFIGURE FOR DIGITAL IO
	MOVWF	ADCON1			;STORE SETTINGS IN AD REGISTER
	CLRF	TRISA			;PORTA ALL OUTPUTS
	CLRF	TRISB			;PORTB ALL OUTPUTS
	BSF		TRISB,HALL		;SET HALL IO BIT TO INPUT PORT B
	BSF		TRISB,MENUKEY	;SET MENU KEY LINE AS INPUT PORT B
	BSF		TRISA,UPKEY		;SET UP KEY AS INPUT PORT A
	BSF		TRISA,DOWNKEY	;SET DOWN KEY AS INPUT PORT A
	BCF		INTCON2,RBPU	;ENABLE WEAK PULLUPS ON PORT B
	CLRF	ROTOR_COUNT		;CLEAR ROTATION COUNTER 0-7 7=THE FULL ROTATION OR 8 MAGNET PASSES ON THE HALL
	MOVLW	B'10000001'		;TIMER0 1:4 PRESCALER 16 BIT TIMER AND ENABLE TIMER
	MOVWF	T0CON
	CLRF 	TMR0H			;CLEAR THE TIMER HIGH BYTE
	CLRF 	TMR0L			;CLEAR THE TIMER LOW BYTE
	BCF		INTCON,TMR0IF	;CLEAR TIMER 0 OVERFLOW
	BSF		INTCON,TMR0IE	;ENABLE TIMER 0 OVERFLOW INTERUPT
	MOVLW	B'10000100'		;TIMER1 1:1 PRESCALER, TIMER DISABLED AND USES INTERNAL CLOCK
	MOVWF	T1CON
	MOVFF	PWM_ON_HIGH,TMR1H ;TRANSFER INTO TIMER1 HIGH REGISTER
	MOVFF	PWM_ON_LOW,TMR1L  ;TRANSFER INTO TIMER1 LOW REGISTER
	BCF		PIR1,TMR1IF		;CLEAR TIMER 1 OVERFLOW
	BSF		PIE1,TMR1IE		;ENABLE TIMER 1 OVERFLOW INTERUPT
	MOVLW	PWM_ON_PRE_HIGH	
	MOVWF	PWM_ON_HIGH		;PRELOAD PULSE ON PERIOD STARTING POINT, THIS WILL ALTER UNTIL THE REQUIRED RPM IS REACHED LATER ON
	MOVLW	PWM_ON_PRE_LOW
	MOVWF	PWM_ON_LOW		;PRELOAD THE LOW BYTE
	MOVLW	RPM_SP_PRE_HIGH ;PRE LOAD OUR STARTING RPM SETPOINT WHICH WE CAN LATER ADJUST USING UP/DOWN KEYS HIGH BYTE
	MOVWF	RPM_SETPOINT_HIGH
	MOVLW	RPM_SP_PRE_LOW  ;PRE LOAD OUR STARTING RPM SETPOINT WHICH WE CAN LATER ADJUST USING UP/DOWN KEYS LOW BYTE
	MOVWF	RPM_SETPOINT_LOW
;PWM INITIALIZATION IF USED
 IF PWM==1
	BSF		TRISB,FETGATE	;PWM INIT MUST OCCUR WITH THE IO LINE SET TO INPUT OTHERWISE INTERNAL DAMAGE CAN OCCUR.
	MOVLW	PWM_PR2
	MOVWF	PR2				;SET THE PWM PERIOD FROM OUR PRESET VALUE
	MOVLW	PWM_CCP1CON		;SET FOR 1 OUTPUT ACTIVE HIGH
	MOVWF	CCP1CON
	MOVLW	PWM_CCPR1L		;SETS THE DUTY CYCLE
	MOVWF	CCPR1L
	CLRF	ECCPAS			;CLEAR THE AUTO SHUTDOWN FEATURE DEFINES THE SHUTDOWN LOGIC STATE OF THE FET PIN
	CLRF	PWM1CON			;SET FOR AUTO RESTART OFF
	BCF		PIR1,TMR2IF		;CLEAR TIMER 2 INTERRUPT FLAG
	MOVLW	PWM_T2CON		;SETS UP TIMER 2
	BSF		W,TMR2ON		;MAKE SURE TIMER2 IS ON
	MOVWF	T2CON			;STORE IN TIMER 2
tim2loop
	BTFSS	PIR1,TMR2IF		;WAIT UNTIL TIMER2 OVERFLOWS BEFORE WE ENABLE OUR PWM OUTPUT PINS
	GOTO	tim2loop
	BCF		TRISB,FETGATE	;WE CAN NOW TURN OUR PWM DRIVEN FET PIN TO AN OUTPUT PIN
	BCF		ECCPAS,ECCPASE	;SHUTDOWN PWM UNTIL WE NEED IT
 ENDIF	
	BSF		INTCON,RBIE		;ENABLE PORT B PIN CHANGE
	BSF 	INTCON,GIE 		;ENABLE INTERRUPTS

;FROM HERE ON WE ARE MONITORING RPM COUNT USING INTERRUPTS THE DURATION THAT THE TIME 8 MAGNETS PASS OR 1 REV OF THE ROTOR TAKES.
;THIS IS UPDATED IN THE RPM_HIGH AND RPM_LOW 16 BIT COUNTER, THE HIGHER THE NUMBER THE SLOWER THE ROTOR VALUES ARE IN HEX.
;3000 RPM = 04EDH     225 RPM = 41A3H   100 RPM = 941AH
;PWM_ON_HIGH & PWM_ON_LOW CONTAIN THE COUNTER VALUE FOR OUR FET DRIVER ON PERIOD, THIS HAPPENS EACH TIME WE GET A LOW FROM THE HALL SENSOR
;WE WANT TO NOW START AT A PRE DEFINED PULSE ON PERIOD AND SLOWLY INCREASE/DECREASE UNTIL OUR RPM SETPOINT IS REACHED.
	
;SO FIRST COMPARE THE RPM TO THE REQUIRED RPM LOOK FOR BEING EQUAL
rpm_loop
	MOVF 	RPM_SETPOINT_HIGH,W
	SUBWF 	RPM_HIGH,W			;X=SETPOINT, Y=RPM
	BTFSS 	STATUS,C			;X>Y..zc,X<Y..zC,X=Y..ZC
	GOTO 	setpoint_is_greater	;RESULT: X>Y
	BTFSS 	STATUS,Z
	GOTO 	rpm_is_greater		;RESULT: Y>X
	MOVF 	RPM_SETPOINT_LOW,W
	SUBWF 	RPM_LOW,W
	BTFSS 	STATUS,C			;X>Y..zc,X<Y..zC,X=Y..ZC
	GOTO 	setpoint_is_greater ;RESULT: X>Y
	BTFSS 	STATUS,Z
	GOTO 	rpm_is_greater		;RESULT: Y>X
	CALL	delay_100uS
	GOTO 	check_keys			;RESULT: X=Y
;RPM TIME IS GREATER THAN SETPOINT SO GOING TOO SLOW, WE NEED TO INCREASE OUR ON TIME BY DECREASING OUR TIMER VALUE
rpm_is_greater
	DECF 	PWM_ON_LOW,F
	BTFSS 	STATUS,C
	DECF 	PWM_ON_HIGH,F
	BTFSS 	STATUS,C
	DECF 	PWM_ON_HIGH,F
	GOTO	check_endstops
;SETPOINT TIME IS GREATER THAN RPM SO GOING TOO FAST, WE NEED TO DECREASE OUR ON TIME BY INCREASING OUR TIMER VALUE
setpoint_is_greater
 	INFSNZ  PWM_ON_LOW,F
   	INCF	PWM_ON_HIGH,F
;OK SO WE HAVE INCREMENTED OR DECREMENTED OUR PWM ON TIME TO MATCH THE SETPOINT ROTOR SPEED AND ACTUAL ROTOR RPM
;NOW WE WANT TO CHECK END STOPS FOR THE ON PERIOD TO MAKE SURE WE DO NOT OVER HEAT THE FET WITH TOO LONG AN ON TIME AND ALSO SO WE DONT ROLL OVER THE 16 BIT TIMER VALUE.

;CHECK FOR ON TIME LIMITS
check_endstops
	MOVLW 	PWM_ON_MIN_HIGH		;THIS SHOULD BE THE HIGHEST LIMITING VALUE
	SUBWF 	PWM_ON_HIGH,W		;X=ON MIN, Y=PWM ON TIME
	BTFSS 	STATUS,C			;X>Y..zc,X<Y..zC,X=Y..ZC
	GOTO 	pwm_on_is_longer	;RESULT: X>Y
	BTFSS 	STATUS,Z
	GOTO 	pwm_on_is_shorter	;RESULT: Y>X
	MOVLW 	PWM_ON_MIN_LOW
	SUBWF 	PWM_ON_LOW,W
	BTFSS 	STATUS,C			;X>Y..zc,X<Y..zC,X=Y..ZC
	GOTO 	pwm_on_is_longer 	;RESULT: X>Y
	BTFSS 	STATUS,Z
	GOTO 	pwm_on_is_shorter	;RESULT: Y>X
	CALL	delay_100uS
	GOTO 	check_keys			;RESULT: X=Y
pwm_on_is_shorter				;WE OVERFLOWED OUR ON TIME IS SHORTER THAN ALLOWED BY OUR LIMIT
	MOVLW	PWM_ON_MIN_HIGH
	MOVWF	PWM_ON_HIGH
	MOVLW	PWM_ON_MIN_LOW
	MOVWF	PWM_ON_LOW
	CALL	delay_100uS
	GOTO	check_keys
pwm_on_is_longer
	MOVLW 	PWM_ON_MAX_HIGH		;THIS SHOULD BE THE LOWEST LIMITING VALUE
	SUBWF 	PWM_ON_HIGH,W		;X=ON MIN, Y=PWM ON TIME
	BTFSS 	STATUS,C			;X>Y..zc,X<Y..zC,X=Y..ZC
	GOTO 	pwm_on_is_longer2	;RESULT: X>Y
	BTFSS 	STATUS,Z
	GOTO 	pwm_on_is_shorter2	;RESULT: Y>X
	MOVLW 	PWM_ON_MAX_LOW
	SUBWF 	PWM_ON_LOW,W
	BTFSS 	STATUS,C			;X>Y..zc,X<Y..zC,X=Y..ZC
	GOTO 	pwm_on_is_longer2 	;RESULT: X>Y
	BTFSS 	STATUS,Z
	GOTO 	pwm_on_is_shorter2	;RESULT: Y>X
	CALL	delay_100uS
	GOTO 	check_keys			;RESULT: X=Y
pwm_on_is_longer2				;WE OVERFLOWED OUR ON TIME IS LONGER THAN ALLOWED BY OUR LIMIT
	MOVLW	PWM_ON_MAX_HIGH
	MOVWF	PWM_ON_HIGH
	MOVLW	PWM_ON_MAX_LOW
	MOVWF	PWM_ON_LOW
pwm_on_is_shorter2	
	CALL	delay_100uS
check_keys
;WE WILL NOW POLL OUR KEYS AND CHECK FOR UP/DOWN TO INCREASE OR DECREASE OUR ROTOR RPM
	BTFSS	PORTA,UPKEY			;CHECK UP KEY LOOKING FOR ACTIVE LOW
	GOTO	up_keypress
	BTFSS	PORTA,DOWNKEY		;CHECK DOWN KEY LOOKING FOR ACTIVE LOW
	GOTO	down_keypress
	BTFSS	PORTB,MENUKEY		;CHECK FOR MENU KEY PRESS
	GOTO	menu_keypress
	GOTO	rpm_loop
down_keypress
;WE WANT TO DECREASE OUR ROTOR RPM BY INCREASING THE TIMER VALUE
 	INFSNZ  RPM_SETPOINT_LOW,F
   	INCF	RPM_SETPOINT_HIGH,F
	GOTO	check_rpmstopsmax
up_keypress
;WE WANT TO INCREASE OUR ROTOR RPM BY DECREASING THE TIMER VALUE
	DECF 	RPM_SETPOINT_LOW,F
	BTFSS 	STATUS,C
	DECF 	RPM_SETPOINT_HIGH,F
	BTFSS 	STATUS,C
	DECF 	RPM_SETPOINT_HIGH,F
	GOTO	check_rpmstopsmin
menu_keypress
;WE DONT USE THE MENU KEY AT THE MOMENT SO RETURN ROUND
	GOTO	rpm_loop
check_rpmstopsmax
;CHECK FOR RPM SETPOINT BOUNDARIES, KEY PRESS UP INCREASES THE SETPOINT VALUE SO WE WANT TO BE BELOW OUR MAX SETPOINT VALUE
	MOVLW 	RPM_SP_MAX_HIGH			;
	SUBWF 	RPM_SETPOINT_HIGH,W		;X=SETPOINT_MAX, Y=SETPOINT
	BTFSS 	STATUS,C				;X>Y..zc,X<Y..zC,X=Y..ZC
	GOTO 	setpoint_is_smaller	;RESULT: X>Y
	BTFSS 	STATUS,Z
	GOTO 	setpoint_is_larger	;RESULT: Y>X
	MOVLW 	RPM_SP_MAX_LOW
	SUBWF 	RPM_SETPOINT_LOW,W
	BTFSS 	STATUS,C				;X>Y..zc,X<Y..zC,X=Y..ZC
	GOTO 	setpoint_is_smaller ;RESULT: X>Y
	BTFSS 	STATUS,Z
	GOTO 	setpoint_is_larger	;RESULT: Y>X
	GOTO	rpm_loop
setpoint_is_smaller
	GOTO	check_rpmstopsmin
;IF HERE WE HAVE EXCEEDED OUR MAX SETPOINT VALUE AND NEED TO RELOAD OUR TOP BOUNDARY VALUES TO STOP THE RPM FROM RUNNING AWAY
setpoint_is_larger
	BSF		PORTA,1
	MOVLW	RPM_SP_MAX_HIGH	
	MOVWF	RPM_SETPOINT_HIGH
	MOVLW	RPM_SP_MAX_LOW	
	MOVWF	RPM_SETPOINT_LOW
	GOTO	rpm_loop
check_rpmstopsmin
;NOT WORRIED BECAUSE WE WERE LOWER THAN OUR SETPOINT MAX VALUE SO NOW NEED TO CHECK OUR MIN VALUE BOUNDARY
	MOVLW 	RPM_SP_MIN_HIGH 		;
	SUBWF 	RPM_SETPOINT_HIGH,W		;X=SETPOINT_MIN, Y=SETPOINT
	BTFSS 	STATUS,C				;X>Y..zc,X<Y..zC,X=Y..ZC
	GOTO 	setpoint_is_smaller2	;RESULT: X>Y
	BTFSS 	STATUS,Z
	GOTO 	setpoint_is_larger2	;RESULT: Y>X
	MOVLW 	RPM_SP_MIN_LOW
	SUBWF 	RPM_SETPOINT_LOW,W
	BTFSS 	STATUS,C				;X>Y..zc,X<Y..zC,X=Y..ZC
	GOTO 	setpoint_is_smaller2 ;RESULT: X>Y
	BTFSS 	STATUS,Z
	GOTO 	setpoint_is_larger2	;RESULT: Y>X
	GOTO	rpm_loop
;IF SETPOINT MIN IS GREATER THAN OUR SETPOINT BOUNDARY WE WANT TO RELOAD OUR MIN BOUNDARY VALUE
setpoint_is_smaller2
 	MOVLW	RPM_SP_MIN_HIGH	
	MOVWF	RPM_SETPOINT_HIGH
	MOVLW	RPM_SP_MIN_LOW	
	MOVWF	RPM_SETPOINT_LOW
	GOTO	rpm_loop
;NOT WORRIED ABOUT BEING HERE ALL IS WELL WE ARE WITHIN OUR MAX AND MIN BOUNDARIES
setpoint_is_larger2
	GOTO	rpm_loop
;--------------------------------------------------------------------------------;
; -------VARIOUS DELAY ROUTINES **NOTE NON OF THESE HAVE BEEN CHECKED FOR ACURACY;
;--------------------------------------------------------------------------------;
delay_250mS			
	MOVLW	0X03
	MOVWF	D1
	MOVLW	0X18
	MOVWF	D2
	MOVLW	0X02
	MOVWF	D3
	GOTO	delay_0
delay_300mS			
	MOVLW	0XD1
	MOVWF	D1
	MOVLW	0X4F
	MOVWF	D2
	MOVLW	0X02
	MOVWF	D3
	GOTO	delay_0
delay_1S			
	MOVLW	0X11
	MOVWF	D1
	MOVLW	0X5D
	MOVWF	D2
	MOVLW	0X05
	MOVWF	D3
	GOTO	delay_0
delay_100uS			
	MOVLW	0XA0
	MOVWF	D1
	MOVLW	0X02
	MOVWF	D2
	MOVLW	0X01
	MOVWF	D3
	GOTO	delay_0
delay_5mS			
	MOVLW	0XB0
	MOVWF	D1
	MOVLW	0X22
	MOVWF	D2
	MOVLW	0X01
	MOVWF	D3
	GOTO	delay_0
delay_40mS			
	MOVLW	0XDA
	MOVWF	D1
	MOVLW	0X0C
	MOVWF	D2
	MOVLW	0X02
	MOVWF	D3
	GOTO	delay_0
delay_0
	DECFSZ	D1, F
	BRA		delay_0
	DECFSZ	D2, F
	BRA		delay_0
	DECFSZ	D3, F
	BRA		delay_0
	RETURN
	END